#include		"windows.h"
#include		"sq.h"

#include		"KernelPlugin.h"
#include		"PlugManPlugin.h" 
#include		"ModManPlugin.h" 
#include		"PlatformPlugin.h"
#include		"WindowPlugin.h"


#define longAt(i) (*((int *) (i)))
#define longAtput(i, val) (*((int *) (i)) = val)

const TCHAR U_SLASH[] = TEXT("/");				//	From sqWin32Window.c
const TCHAR U_BACKSLASH[] = TEXT("\\");		// 	From sqWin32Window.c


static struct KernelProxy					*kernelProxy ;
static struct PlatformProxy				*platformProxy ;
static struct WindowProxy				 	*windowProxy ;
static struct PlugManProxy				*plugManProxy ;
static struct VirtualMachine			*interpreterProxy ;
static struct ModManProxy					*modManProxy ;

#if		1			// Moved to Startup
/*	Return the next 4-byte word of the given file, byte-swapped according to the given flag. */

int getLongFromFileswap(sqImageFile f, int swapFlag) {
    int w;

	platformProxy->sqImageFileRead(&w, sizeof(char), 4, f);
	if (swapFlag) {
		return ((((((unsigned) w >> 24)) & 255) + ((((unsigned) w >> 8)) & 65280)) + ((((unsigned) w << 8)) & 16711680)) + ((((unsigned) w << 24)) & 4278190080U);
	} else {
		return w;
	}
}
#endif					// Moved to Startup

#if		1			// Moved to Startup
/*	Read and verify the image file version number and return true if the the given image file needs to be byte-swapped. As a side effect, position the file stream just after the version number of the image header. This code prints a warning and does a hard-exit if it cannot find a valid version number. */
/*	This code is based on C code by Ian Piumarta. */

int checkImageVersionFromstartingAt(sqImageFile f, int imageOffset) {
    int version;
    int firstVersion;

	platformProxy->sqImageFileSeek(f, imageOffset);
	version = firstVersion = getLongFromFileswap(f, 0);
	if (version == 6502) {
		return 0;
	}
	
	platformProxy->sqImageFileSeek(f, imageOffset);
	version = getLongFromFileswap(f, 1);
	if (version == 6502) {
		return 1;
	}
	if (imageOffset == 0) {
		platformProxy->sqImageFileSeek(f, 512);
		version = getLongFromFileswap(f, 0);
		if (version == 6502) {
			return 0;
		}
		platformProxy->sqImageFileSeek(f, 512);
		version = getLongFromFileswap(f, 1);
		if (version == 6502) {
			return 1;
		}
	}
	interpreterProxy->print("This interpreter (vers. ");
	interpreterProxy->printNum(6502);
	interpreterProxy->print(" cannot read image file (vers. ");
	interpreterProxy->printNum(firstVersion);
	/* begin cr */
	printf("\n");
	interpreterProxy->print("Hit CR to quit");
	getchar();
	platformProxy->ioExit();
}
#endif					// Moved to Startup

#if		1			// Moved from InterpreterPlugin
/*	Initialize object memory variables at startup time. Assume endOfMemory is initially set (by the image-reading code) to the end of the last object in the image. Initialization redefines endOfMemory to be the end of the object allocation area based on the total available memory, but reserving some space for forwarding blocks. */
/*	Assume: image reader initializes the following variables:
		memory
		endOfMemory
		memoryLimit
		specialObjectsOop
		lastHash
	 */
/*	di 11/18/2000 fix slow full GC */

int initializeObjectMemory(int bytesToShift) {
    int header;
    int oop;
    int totalObjects;
    int chunk;
    int extra;
    int fieldOop;
    int newClassOop;
    int fieldAddr;
    int classHeader;
    int extra1;
    int type;
    int sz;
    int extra2;
    int header1;
    int extra11;
    int type1;
		int	specialObjectsOop ;

		int	endOfMemory ;

	/* image may be at a different address; adjust oops for new location */

	interpreterProxy->setYoungStart(interpreterProxy->getEndOfMemory()) ;
	/* begin adjustAllOopsBy: */
	totalObjects = 0;
	/* begin oopFromChunk: */
	chunk = interpreterProxy->startOfMemory();
	/* begin extraHeaderBytes: */
	type = (longAt(chunk)) & 3;
	if (type > 1) {
		extra1 = 0;
	} else {
		if (type == 1) {
			extra1 = 4;
		} else {
			extra1 = 8;
		}
	}
	extra = extra1;
	endOfMemory = interpreterProxy->getEndOfMemory() ;

	oop = chunk + extra;
	while (oop < endOfMemory) {
		if (!(((longAt(oop)) & 3) == 2)) {
			totalObjects += 1;
			header = longAt(oop);
			longAtput(oop, header & 3221225471U);
			/* begin adjustFieldsAndClassOf:by: */
			fieldAddr = oop + (interpreterProxy->lastPointerOf(oop));
			while (fieldAddr > oop) {
				fieldOop = longAt(fieldAddr);
				if (!((fieldOop & 1))) {
					longAtput(fieldAddr, fieldOop + bytesToShift);
				}
				fieldAddr -= 4;
			}
			if (((longAt(oop)) & 3) != 3) {
				classHeader = longAt(oop - 4);
				newClassOop = (classHeader & 4294967292U) + bytesToShift;
				longAtput(oop - 4, newClassOop | (classHeader & 3));
			}
		}
		/* begin objectAfter: */
		;
		if (((longAt(oop)) & 3) == 2) {
			sz = (longAt(oop)) & 4294967292U;
		} else {
			/* begin sizeBitsOf: */
			header1 = longAt(oop);
			if ((header1 & 3) == 0) {
				sz = (longAt(oop - 8)) & 4294967292U;
				goto l1;
			} else {
				sz = header1 & 252;
				goto l1;
			}
		l1:	/* end sizeBitsOf: */;
		}
		/* begin oopFromChunk: */
		/* begin extraHeaderBytes: */
		type1 = (longAt(oop + sz)) & 3;
		if (type1 > 1) {
			extra11 = 0;
		} else {
			if (type1 == 1) {
				extra11 = 4;
			} else {
				extra11 = 8;
			}
		}
		extra2 = extra11;
		oop = (oop + sz) + extra2;
	}
	interpreterProxy->setTotalObjectCount(totalObjects) ;
	interpreterProxy->initializeMemoryFirstFree(endOfMemory);

	/* heavily used special objects */

	specialObjectsOop = interpreterProxy->getSpecialObjectsOop() ;
	specialObjectsOop += bytesToShift;
	interpreterProxy->setSpecialObjectsOop(specialObjectsOop) ;

	interpreterProxy->setNilObj(longAt(((((char *) specialObjectsOop)) + 4) + (0 << 2))) ;
	interpreterProxy->setFalseObj(longAt(((((char *) specialObjectsOop)) + 4) + (1 << 2))) ;
	interpreterProxy->setTrueObj(longAt(((((char *) specialObjectsOop)) + 4) + (2 << 2))) ;
	interpreterProxy->setRootTableCount(0) ;
	interpreterProxy->setChild(0) ;
	interpreterProxy->setField(0) ;
	interpreterProxy->setParentField(0) ;
	interpreterProxy->setFreeContexts(1) ;
	interpreterProxy->setFreeLargeContexts(1) ;
	interpreterProxy->setAllocationCount(0) ;
	interpreterProxy->setLowSpaceThreshold(0) ;
	interpreterProxy->setSignalLowSpace(0) ;
	interpreterProxy->setCompStart(0) ;
	interpreterProxy->setCompEnd(0) ;
	interpreterProxy->setFwdTableNext(0) ;
	interpreterProxy->setFwdTableLast(0) ;
	interpreterProxy->setRemapBufferCount(0) ;

	/* do incremental GC after this many allocations */

	interpreterProxy->setAllocationsBetweenGCs(4000) ;

	/* tenure all suriving objects if count is over this threshold */

	interpreterProxy->setTenuringThreshold(2000) ;

	/* four megabyte of headroom when growing */

	interpreterProxy->setGrowHeadroom((4 * 1024) * 1024) ;

	/* eight megabyte of free space before shrinking */
	/* garbage collection statistics */

	interpreterProxy->setShrinkThreshold((8 * 1024) * 1024) ;
	interpreterProxy->setStatFullGCs(0) ;
	interpreterProxy->setStatFullGCMSecs(0) ;
	interpreterProxy->setStatIncrGCs(0) ;
	interpreterProxy->setStatIncrGCMSecs(0) ;
	interpreterProxy->setStatTenures(0) ;
	interpreterProxy->setStatRootTableOverflows(0) ;
	interpreterProxy->setDisplayBits(0) ;
}
#endif					// Moved from InterpreterPlugin


#if		1			// Moved from InterpreterPlugin
/*	Read an image from the given file stream, allocating the given amount of memory to its object heap. Fail if the image has an unknown format or requires more than the given amount of memory. */
/*	Details: This method detects when the image was stored on a machine with the opposite byte ordering from this machine and swaps the bytes automatically. Furthermore, it allows the header information to start 512 bytes into the file, since some file transfer programs for the Macintosh apparently prepend a Mac-specific header of this size. Note that this same 512 bytes of prefix area could also be used to store an exec command on Unix systems, allowing one to launch Smalltalk by invoking the image name as a command. */
/*	This code is based on C code by Ian Piumarta and Smalltalk code by Tim Rowledge. Many thanks to both of you!! */

int readImageFromFileHeapSizeStartingAt(sqImageFile f, int desiredHeapSize, int imageOffset) {
  int memStart;
  int heapSize;
  int swapBytes;
  int bytesRead;
  int oldBaseAddr;
  int headerSize;
  int minimumMemory;
  int headerStart;
  int bytesToShift;
  int dataSize;
  int i;
  int startAddr;
  int addr;
  int proc;
  int sched;
  int activeCntx;
  int tmp;

	int	memory ;
	int	nilObj ;
	int	endOfMemory ;

	swapBytes = checkImageVersionFromstartingAt(f, imageOffset);

	/* record header start position */

	headerStart = (platformProxy->sqImageFilePosition(f)) - 4;
	headerSize = getLongFromFileswap(f, swapBytes);
	dataSize = getLongFromFileswap(f, swapBytes);
	oldBaseAddr = getLongFromFileswap(f, swapBytes);
	interpreterProxy->setSpecialObjectsOop(getLongFromFileswap(f, swapBytes)) ;
	interpreterProxy->setLastHash(getLongFromFileswap(f, swapBytes)) ;
	interpreterProxy->setSavedWindowSize(getLongFromFileswap(f, swapBytes)) ;
	interpreterProxy->setFullScreenFlag(getLongFromFileswap(f, swapBytes)) ;
	interpreterProxy->setExtraVMMemory(getLongFromFileswap(f, swapBytes)) ;

	if (0 == interpreterProxy->getLastHash()) {
		interpreterProxy->setLastHash(999) ;
	}
	heapSize = reserveExtraCHeapBytes(desiredHeapSize, extraVMMemory);

	/* need at least 100K of breathing room */

	minimumMemory = dataSize + 100000;
	if (heapSize < minimumMemory) {
				interpreterProxy->error("Insufficient memory for this image");
;
	}
	memory = (unsigned char *) interpreterProxy->sqAllocateMemory(minimumMemory, heapSize);
	if (memory == null) {
				interpreterProxy->error("Failed to allocate memory for the heap");
;
	}
	
	interpreterProxy->setMemory(memory) ;
	
	memStart = interpreterProxy->startOfMemory();

	/* decrease memoryLimit a tad for safety */

	interpreterProxy->setMemoryLimit((memStart + heapSize) - 24) ;

	/* position file after the header */

	interpreterProxy->setEndOfMemory(memStart + dataSize) ;
	platformProxy->sqImageFileSeek(f, headerStart + headerSize);
	bytesRead = platformProxy->sqImageFileRead(memory, sizeof(unsigned char), dataSize, f);
	if (bytesRead != dataSize) {
				interpreterProxy->error("Read failed or premature end of image file");
;
	}
	if (swapBytes) {
		/* begin reverseBytesInImage */
		/* begin reverseBytesFrom:to: */
		startAddr = interpreterProxy->startOfMemory();
		addr = startAddr;
		
		endOfMemory = interpreterProxy->getEndOfMemory() ;

		while (addr < endOfMemory) {
			longAtput(addr, ((((((unsigned) (longAt(addr)) >> 24)) & 255) + ((((unsigned) (longAt(addr)) >> 8)) & 65280)) + ((((unsigned) (longAt(addr)) << 8)) & 16711680)) + ((((unsigned) (longAt(addr)) << 24)) & 4278190080U));
			addr += 4;
		}
		/* begin byteSwapByteObjects */
		interpreterProxy->byteSwapByteObjectsFromto(interpreterProxy->oopFromChunk(interpreterProxy->startOfMemory()), endOfMemory);
	}
	bytesToShift = memStart - oldBaseAddr;
	/* begin initializeInterpreter: */
//	interpreterProxy = sqGetInterpreterProxy();
	initializeObjectMemory(bytesToShift);
	interpreterProxy->initCompilerHooks();
	interpreterProxy->flushExternalPrimitives();

	nilObj = interpreterProxy->getNilObj() ;

	interpreterProxy->setActiveContext(nilObj) ;
	interpreterProxy->setTheHomeContext(nilObj) ;
	interpreterProxy->setMethod(nilObj) ;
	interpreterProxy->setReceiver(nilObj) ;
	interpreterProxy->setMessageSelector(nilObj) ;
	interpreterProxy->setNewMethod(nilObj) ;
	interpreterProxy->setMethodClass(nilObj) ;
	interpreterProxy->setLkupClass(nilObj) ;
	interpreterProxy->setReceiverClass(nilObj) ;

	/* begin flushMethodCache */
	for (i = 1; i <= 4096; i += 1) {
		interpreterProxy->methodCache[i] = 0;
	}
	for (i = 1; i <= 64; i += 1) {
		interpreterProxy->atCache[i] = 0;
	}
	/* begin loadInitialContext */
	sched = longAt(((((char *) (longAt(((((char *) interpreterProxy->getSpecialObjectsOop())) + 4) + (3 << 2))))) + 4) + (1 << 2));
	proc = longAt(((((char *) sched)) + 4) + (1 << 2));
	interpreterProxy->setActiveContext(longAt(((((char *) proc)) + 4) + (1 << 2))) ;
	if (interpreterProxy->getActiveContext() < interpreterProxy->getYoungStart()) {
		interpreterProxy->beRootIfOld(interpreterProxy->getActiveContext());
	}
	/* begin fetchContextRegisters: */
	activeCntx = interpreterProxy->getActiveContext() ;
	tmp = longAt(((((char *) activeCntx)) + 4) + (3 << 2));
	if ((tmp & 1)) {
		tmp = longAt(((((char *) activeCntx)) + 4) + (5 << 2));
		if (tmp < interpreterProxy->getYoungStart()) {
			interpreterProxy->beRootIfOld(tmp);
		}
	} else {
		tmp = activeCntx;
	}
	interpreterProxy->setTheHomeContext(tmp) ;
	interpreterProxy->setReceiver(longAt(((((char *) tmp)) + 4) + (5 << 2))) ;
	interpreterProxy->setMethod(longAt(((((char *) tmp)) + 4) + (3 << 2))) ;
	tmp = ((longAt(((((char *) activeCntx)) + 4) + (1 << 2))) >> 1);
	interpreterProxy->setInstructionPointer(((interpreterProxy->getMethod() + tmp) + 4) - 2) ;
	tmp = ((longAt(((((char *) activeCntx)) + 4) + (2 << 2))) >> 1);
	interpreterProxy->setStackPointer((activeCntx + 4) + (((6 + tmp) - 1) * 4)) ;
	interpreterProxy->setReclaimableContextCount(0) ;
	interpreterProxy->setInterruptCheckCounter(0) ;
	interpreterProxy->setInterruptCheckCounterFeedBackReset(1000) ;
	interpreterProxy->setInterruptChecksEveryNms(5) ;
	interpreterProxy->setNextPollTick(0) ;
	interpreterProxy->setNextWakeupTick(0) ;
	interpreterProxy->setLastTick(0) ;
	interpreterProxy->setInterruptKeycode(2094) ;
	interpreterProxy->setInterruptPending(0) ;
	interpreterProxy->setSemaphoresUseBufferA(1) ;
	interpreterProxy->setSemaphoresToSignalCountA(0) ;
	interpreterProxy->setSemaphoresToSignalCountB(0) ;
	interpreterProxy->setDeferDisplayUpdates(0) ;
	interpreterProxy->setPendingFinalizationSignals(0) ;
	return dataSize;
}
#endif					// Moved from InterpreterPlugin


#if			1		// from sqWin32Window.c
/* SqueakImageLength(): 
   Return the length of the image if it is a valid Squeak image file.
   Otherwise return 0. */
DWORD SqueakImageLength(TCHAR *fileName)
{ DWORD dwRead, dwSize, magic = 0;
  HANDLE hFile;

	if(NULL == platformProxy)
		platformProxy = kernelProxy->ioGetPluginProxy("PlatformPlugin") ;
	

  /* open image file */
  hFile = CreateFile(fileName, GENERIC_READ, FILE_SHARE_READ, 
                     NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if(hFile == INVALID_HANDLE_VALUE) 
    return 0;
  /* read magic number */
  if(!ReadFile(hFile, &magic, 4, &dwRead, NULL))
    {
      CloseHandle(hFile);
      return 0;
    }
  /* get the image's size */
  dwSize = GetFileSize(hFile, NULL);
  CloseHandle(hFile);
  if(!platformProxy->readableFormat(magic) && !platformProxy->readableFormat(platformProxy->byteSwapped(magic)))
    return 0;
  else
    return dwSize;
}
#endif					// from sqWin32Window.c

#if				1  // from sqWin32Window.c
/****************************************************************************/
/*                      Startup helper functions                            */
/****************************************************************************/

/* findImageFile():
   Search the current directory for exactly one image file.
   If it is found, copy the name into imageName and return true.
*/
int findImageFile(void)
{
  WIN32_FIND_DATA findData;
  HANDLE					findHandle;
  int							nextFound;
	char						*imageName ;

	if(NULL == platformProxy)
		platformProxy = kernelProxy->ioGetPluginProxy("PlatformPlugin") ;
	

	imageName = kernelProxy->imageName ;

  findHandle = FindFirstFile(TEXT("*.image"),&findData);
  
	if(findHandle == INVALID_HANDLE_VALUE)
	  return 0; /* Not found */
  
	nextFound = FindNextFile(findHandle,&findData);
  FindClose(findHandle);
  
	if(nextFound) return 0; /* more than one entry */
  
	strcpy(imageName, platformProxy->fromUnicode(findData.cFileName));
  return 1;
}

// from sqWin32Window.c
/* openImageFile():
   Pop up a file open dialog for image files.
   Copy the selection into imageName and return true.
*/
int openImageFile(void)
{
  OPENFILENAME	ofn;
	TCHAR					tmpImagePathName[MAX_PATH] ;

	if(NULL == platformProxy)
		platformProxy = kernelProxy->ioGetPluginProxy("PlatformPlugin") ;

	tmpImagePathName[0] = 0 ;

  memset(&ofn, 0, sizeof(ofn));
  ofn.lStructSize = sizeof(ofn);
  ofn.lpstrFilter = TEXT("Image Files (*.image)\0*.image\0All Files (*.*)\0*.*\0");
  ofn.lpstrFile = tmpImagePathName;
  ofn.nMaxFile = sizeof(tmpImagePathName);
  ofn.lpstrTitle = TEXT("SQUEAK: Please select an image file...");
  ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
  ofn.lpstrDefExt = TEXT("image");
  MessageBeep(MB_ICONSTOP);
  if (!GetOpenFileName(&ofn)) return 0;
  strcpy(kernelProxy->imageName, platformProxy->fromUnicode(tmpImagePathName));
  return 1;
}
#endif				  // from sqWin32Window.c


#if			1			// Moved from PlatformPlugin
/****************************************************************************/
/*                      Embedded Images                                     */
/****************************************************************************/
#if 0
/* SQ_IMAGE_MAGIC - the magic number for embedded images "SQIM" */
#define SQ_IMAGE_MAGIC 0x83817377

int sqImageFile findEmbeddedImage(void) {
	sqImageFile f;
	int endMarker;
	int magic;
	int start;
	int length;

	f = sqImageFileOpen(vmName, "rb");
	if(!f) {
		MessageBox(0,"Error opening VM","Squeak",MB_OK);
		return 0;
	}
	endMarker = sqImageFileSize(f) - 8;
	magic = start = 0;
	sqImageFileSeek(f, endMarker);
    sqImageFileRead(&magic, 1, 4, f);
	sqImageFileRead(&start, 1, 4, f);
	sqMessageBox(MB_OK, "Magic number", "Expected:\t%x\nFound:\t\t%x", SQ_IMAGE_MAGIC, magic);
	/* Magic number must be okay and start must be within executable boundaries */
	if(magic != SQ_IMAGE_MAGIC || start < 0 || start >= endMarker) {
		/* nope */
		sqImageFileClose(f);
		return 0;
	}
	/* Might have an embedded image; seek back and double check */
	sqImageFileSeek(f,start);
	sqImageFileRead(&magic, 1, 4, f);
	sqMessageBox(MB_OK, "Magic number", "Expected:\t%x\nFound:\t\t%x", SQ_IMAGE_MAGIC, magic);
	if(magic != SQ_IMAGE_MAGIC) {
		/* nope */
		sqImageFileClose(f);
		return 0;
	}
	/* now triple check for image format */
	sqImageFileRead(&magic, 1, 4, f);
	if(!readableFormat(magic) && !readableFormat(byteSwapped(magic)) {
		/* nope */
		sqImageFileClose(f);
		return 0;
	}
	/* Gotcha! */
	sqImageFileSeek(f, sqImageFilePosition(f) - 4);
	strcpy(kernelProxy->imageName, name);
	imageSize = endMarker - sqImageFilePosition(f);
	return f;
}
#else
int findEmbeddedImage(void) { return 0; }
#endif
#endif						// Moved from PlatformPlugin


#if			1			// Moved from InterpreterPlugin
void initObjectMemory(void)
{
	sqImageFile imageFile = 0 ;
	int					imageSize = 0;
	int					filePosition ;
  int					virtualMemory;			// Moved from sqWIn32Intel.c

	TCHAR				*tmp;
  TCHAR				tmpName[MAX_PATH+1];
  TCHAR				imageNameW[MAX_PATH+1];
	char				*imageName ;
	TCHAR				*imagePath ;

	if(NULL == platformProxy)
		platformProxy = kernelProxy->ioGetPluginProxy("PlatformPlugin") ;
	
	if(NULL == interpreterProxy)
		interpreterProxy = kernelProxy->ioGetPluginProxy("InterpreterPlugin") ;

	
//	imageFile = platformProxy->getImageFileHandle() ;


#if			1			// Moved to PlatformPlugin
  /* a quick check if we have any argument at all */
  if(!platformProxy->getRunServiceFlag() && (*(kernelProxy->imageName) == 0)) 
	{
    /* Check if the image is embedded */
    imageFile = findEmbeddedImage();
  
		if(!imageFile) 
		{
      /* Search the current directory if there's a single image file */
      if(!findImageFile()) 
			{
				/* Nope. Give the user a chance to open an image interactively */
				if(!openImageFile()) 
					return -1; /* User cancelled file open */
      }
    }
  }
#endif						// Moved to PlatformPlugin


	imagePath = kernelProxy->imagePath ;
	imageName = kernelProxy->imageName ;

#if			1			// Moved from PlatformPlugin
	/* get the full path for the image */
	lstrcpy(tmpName, platformProxy->toUnicode(imageName));
	GetFullPathName(tmpName,MAX_PATH,imageNameW,&tmp);
	/* check if it is a short name and if so convert 
	   it to the full name */
	platformProxy->LongFileNameFromPossiblyShortName(imageNameW);
	
	/* and copy back to a C string */
	strcpy(imageName, platformProxy->fromUnicode(imageNameW));
#endif						// Moved from PlatformPlugin

#if			1			// Moved from PlatformPlugin
  lstrcpy(imagePath, imageName);
  tmp = platformProxy->lstrrchr(imagePath,U_BACKSLASH[0]);
  if(tmp) tmp[1] = 0;
#endif						// Moved from PlatformPlugin


#if		1			// Moved from sqWIn32Intel.c
  if(!imageFile) {
    imageSize = SqueakImageLength(platformProxy->toUnicode(kernelProxy->imageName));
    if(imageSize == 0) platformProxy->printUsage(2);
  }

  /* read the image file */
  if(!imageFile) {
    imageFile = platformProxy->sqImageFileOpen(kernelProxy->imageName,"rb");
		filePosition = 0 ;
  } else {
    filePosition = platformProxy->sqImageFilePosition(imageFile) ;
  }

#ifdef NO_VIRTUAL_MEMORY
  if(!dwMemorySize) {
    dwMemorySize = 4;
    virtualMemory = (int)imageSize + max(imageSize, dwMemorySize * 0x00100000);
  } else {
    virtualMemory = (int)dwMemorySize * 0x00100000;
  }
#else
  /* initial commit size = imageSize + 4MB */
  virtualMemory = imageSize + 0x00400000;
#endif

  readImageFromFileHeapSizeStartingAt(imageFile, virtualMemory, filePosition) ;
  platformProxy->sqImageFileClose(imageFile);

#endif					// From sqWin32Intel.c

	return ;
}
#endif						// Moved from InterpreterPlugin


int initObjectMemoryStartup(struct KernelProxy * aProxy) 
{
	kernelProxy = aProxy ;
	initObjectMemory() ;

	return 1;
}

